package uicc.contactmanager;


/**
 * <p>
 *   A field descriptors list holds all field descriptors defined for a contact manager.
 * </p>
 * <p>
 *   It defines all fields that any contact shall have. Therefore, it represents the structure
 *   of all contacts.
 * </p>
 * <p>
 *   The field descriptors list shall be setup before any contact has been created in the contact manager.
 * </p>
 * <p>
 *   Within a descriptors list, each field descriptor is assigned a numeric identifier,
 *   that is unique within the list.
 *   <br>
 *   The identifier for each field descriptor is assigned by the containing field 
 *   descriptors list and is only significant within the parent contact manager.
 * </p>
 * <p>
 *   An object that realizes the <code>FieldDescriptorsListView</code> interface represents 
 *   a field descriptors list.
 *   <br>
 *   It represents the same field descriptors list since its creation, during its whole lifetime.
 * </p>
 * <p>
 *   Each <code>FieldDescriptorsListView</code> instance provides an independent enumeration scheme 
 *   to enumerate all field descriptors that belong to the list.
 * </p>
 * <p>
 *   <code>FieldDescriptorsListView</code> instances also provide a factory method to create an instance 
 *   of the <code>FieldDescriptorView</code> interface.
 * </p>
 * <p>
 *   Multiple <code>FieldDescriptorsListView</code> instances may be created by the same or different 
 *   applications. Although they may represent the same descriptors list, each of them has its 
 *   own state.
 *   <br>
 *   This particularly applies to the current enumeration progress, which is specific to
 *   each instance.
 * </p>
 */
public interface FieldDescriptorsListView {
    /**
     * <p>
     *   Factory method used to create an instance of the <code>FieldDescriptorView</code> interface.
     * </p>
     * <p>
     *   The instance is initially created in the DESELECTED state.
     *   <br> 
     *   See the description of the <code>FieldDescriptorView</code> interface for detailed information
     *   about its complete life cycle.
     * </p>
     * <p>
     *   The new instance is owned by the calling application.
     * </p>
     *
     * @return new <code>FieldDescriptorView</code> instance
     */
    public FieldDescriptorView createFieldDescriptorViewInstance();

    /**
     * <p>
     *   Starts a new enumeration of all field descriptors in the list.
     * </p>
     * <ul>
     *   <li>A <code>FieldDescriptorsListView</code> instance can process one enumeration at a time.<br>
     *       If an enumeration is already in progress, it is reset to the first descriptor
     *       in the list.</li>
     *   <li>The enumeration is processed using the method <code>nextFieldDescriptor()</code>.</li>
     * </ul>
     */
    public void startEnumeration();

    /**
     * <p>
     *   Checks if there are more field descriptors in the current enumeration.
     * </p>
     * 
     * @return <code>true</code> if there is at least one more descriptor in the current enumeration
     */
    public boolean hasMoreFieldDescriptors();
    
    /**
     * <p>
     *   Binds a <code>FieldDescriptorView</code> instance with the next descriptor in the current enumeration.
     * </p>
     * <p>
     *   The order in which the descriptors are enumerated is unspecified.
     * </p>
     * <p>
     *   After successful operation, the <code>FieldDescriptorView</code> instance is in the SELECTED state, 
     *   and is bound to the next descriptor in the current enumeration.
     * </p>
     * <p>
     *   If the current enumeration has not been started, or already reached its end,
     *   or after unsuccessful operation for any reason, the <code>FieldDescriptorView</code> instance
     *   is in DESELECTED state.
     * </p>
     * 
     * @param fieldDescriptorView <code>FieldDescriptorView</code> instance to bind to the next descriptor 
     *                        in the enumeration.
     * 
     * @return descriptor identifier, or -1 if the current enumeration reached its end.
     */
    public int selectNextFieldDescriptor(FieldDescriptorView fieldDescriptorView);

    /**
     * <p>
     *   Binds a <code>FieldDescriptorView</code> instance to the descriptor that matches the provided 
     *   <code>fieldDescriptorIdentifier</code>.
     * </p>
     * <p>
     *   After successful operation, the <code>FieldDescriptorView</code> instance is in SELECTED state, 
     *   and is bound to the selected descriptor.
     * </p>
     * <p>
     *   After unsuccessful operation, the <code>FieldDescriptorView</code> instance is in DESELECTED state.
     * </p>
     * 
     * @param fieldDescriptorView <code>FieldDescriptorView</code> instance to bind with the next descriptor that
     *                matches the identifier. 
     * @param fieldDescriptorIdentifier descriptor identifier of the field descriptor to get
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code>:
     *    <ul>
     *      <li>with reason <code>WRONG_IDENTIFIER</code> if the list contains no descriptor with the 
     *          <code>fieldDescriptorIdentifier</code> provided.</li>
     *    </ul>
     */
    public void selectFieldDescriptor(FieldDescriptorView fieldDescriptorView, int fieldDescriptorIdentifier);

    /**
     * <p>
     *   Adds a field descriptor to the field descriptors list.
     * </p>
     * <p>
     *   The descriptor to be added is represented by an instance of <code>FieldDescriptorView</code>
     *   that is in DESELECTED state, whose attributes have been set to the appropriate values.
     * </p>
     * <p>
     *   When a descriptor is added to a descriptors list, any enumeration over the same descriptors
     *   list currently in progress may or may not include the new descriptor, depending on the 
     *   implementation.
     * </p>
     * <p>
     *   After successful completion of this operation, the <code>FieldDescriptorView</code> instance is 
     *   in SELECTED state, and is bound to the newly created descriptor.
     * </p>
     *
     * @param fieldDescriptorView A <code>FieldDescriptorView</code> instance with attributes set according 
     *                to appropriate values.
     * 
     * @return the new descriptors's identifier
     */ 
    public int addFieldDescriptor(FieldDescriptorView fieldDescriptorView);

    /**
     * <p>
     *   Removes a field descriptor from the field descriptors list.
     * </p>
     * <p>
     *   This method fails in the following cases:
     *   <ul>
     *     <li>
     *       If any <code>FieldDescriptorView</code> instance is currently bound to the removed descriptor.</li>
     *     <li>
     *       If an enumeration is currently in progress in any instance of <code>FieldDescriptorList</code>
     *       related the same contact manager, other than the method's target.
     *     </li>
     *   </ul>
     * </p>
     * 
     * @param fieldDescriptorIdentifier identifier of the field descriptor to be removed from the list
     * 
     * @throws <code>uicc.contactmanager.ContactManagerException</code> with the following reasons:
     *   <ul>
     *      <li><code>WRONG_IDENTIFIER</code> if <code>fieldDescriptorIdentifier</code>
     *          does not identify any descriptpor in the field descriptors list.</li>
     *      <li><code>ITEM_SELECTED</code> if the descriptor to be removed is currently selected
     *        by a <code>FieldDescriptorView</code> instance.</li>
     *      <li><code>ENUMERATION_IN_PROGRESS</code> if a conflicting enumeration is
     *          currently in progress.</li>
     *   </ul>
     */
    public void removeFieldDescriptor(int fieldDescriptorIdentifier);

    /**
     * Returns the number of field descriptors in the list.
     * 
     * @return number of field descriptors
     */
    public int getNumberOfFieldDescriptors();
}
